unit LwIXFrm;	     {- Incremental eXception Dialog }
interface
uses
  SysUtils, WinTypes, WinProcs, Messages, Classes, Graphics, Controls,
  Forms, Dialogs, StdCtrls, Buttons,
  LwIX;

type
  TOnShowMsgerEvent = function (IX   : EIncrementalException;
				msgr : TEMessager) : boolean of object;
  TOnRecastEvent = function (	 E  : Exception;
			     var IX : EIncrementalException) : boolean of object;

  TIXEngineForm = class (TForm)
    memBoard: TMemo;
    btnOk: TBitBtn;
    btnHelp: TBitBtn;
    btnPrint: TBitBtn;
    dlgPrint: TPrintDialog;
    procedure FormCreate(Sender : TObject);
    procedure FormShow(Sender : TObject);
    procedure btnPrintClick(Sender: TObject);
    procedure btnOkClick(Sender: TObject);
    procedure FormKeyDown(Sender: TObject; var Key: Word; Shift: TShiftState);
    procedure memBoardChange(Sender: TObject);
    procedure FormDestroy(Sender: TObject);
  private
    FAppOnException    : TExceptionEvent;
    FExceptionToShow   : EIncrementalException;
    FEMessagerClass    : TEMessagerClass;
    FInRecastEvent     : boolean;
    FOnRecast	       : TOnRecastEvent;
    FOnShowMsger       : TOnShowMsgerEvent;
    procedure ShowException (Sender : TObject; E : Exception);
      {- to be substituted for Application.OnException }
  public
    procedure RaiseIX (      mcKind	: integer;
		       const msgrParams : array of const;
			     hc 	: THelpContext;
		       var   IX 	: EIncrementalException);
   {Ŀ
     Raises EIncrementalException					      
      - with the first TEMessager of class FEMessagerClass		      
      - that TEMessager corresponds to the other parameters		      
    }
    function  RecastToIX (	E	   : Exception;
				mcKind	   : integer;
			  const msgrParams : array of const;
				hc	   : THelpContext) : boolean;
   {Ŀ
     Returns True iff							      
      - E is EIncrementalException and 				      
      - a new TEMessager, of class FEMessagerClass and according to the other
    	 parameters, was added to E.					      
     Otherwise, calls RaiseIX with the other parameters.		      
    }
    property MessagerClass : TEMessagerClass read FEMessagerClass write FEMessagerClass;
    property ExceptionToShow : EIncrementalException read FExceptionToShow write FExceptionToShow;
    property AppOnException : TExceptionEvent read FAppOnException write FAppOnException;
    property MemoBoard : TMemo read memBoard write memBoard;
    property OnShowMsger : TOnShowMsgerEvent read FOnShowMsger write FOnShowMsger;
    property OnRecast : TOnRecastEvent read FOnRecast write FOnRecast;
  end;

var DefaultIXEngineForm : TIXEngineForm;

implementation {==============================================================}
uses Printers;

  {$R *.DFM}
  {$I SYSUTILS.INC}  {- to have SExceptTitle }

  procedure TIXEngineForm.FormCreate (Sender : TObject);
  begin
    FAppOnException := Application.OnException;
      {- saves the "original" application's OnException handling }
    Application.OnException := ShowException;
      {- sets new exception handling }

    MessagerClass := TEMessager;	       {- default setting }
    Caption := LoadStr (SExceptTitle);	       {- default setting }
  end;

  procedure TIXEngineForm.FormDestroy(Sender: TObject);
  begin
    if Assigned (FAppOnException) then
      Application.OnException := FAppOnException;
      {- restores the "original" application's OnException handling }
  end;

  procedure TIXEngineForm.FormShow(Sender: TObject);
  var errIdx : integer;
      errLin : integer;
      ss     : string;
  begin
    MemoBoard.Clear;
    SetFocusedControl (MemoBoard);
    if ExceptionToShow = nil then Exit;

    with ExceptionToShow do
    begin
      Self.HelpContext := HelpContext;
      btnHelp.HelpContext := HelpContext;
      btnHelp.Enabled := HelpContext <> 0;

      if ErrCount > 0 then
	for errIdx := ErrCount-1 downto 0 do
	if ErrMsgers[errIdx].Count > 0 then
	begin
	  if not Assigned (FOnShowMsger) or
	     OnShowMsger (ExceptionToShow, ErrMsgers[errIdx]) then
	  ErrMsgers[errIdx].AppendToShow (MemoBoard.Lines);
	end;
    end;
    MemoBoard.ShowHint := true; {- if disabled in OnChange }
  end;

  procedure TIXEngineForm.btnPrintClick(Sender: TObject);
  var prnText : System.Text;
      lin     : integer;
  begin
    if dlgPrint.Execute then
    begin
      AssignPrn (prnText);
      try
	Rewrite (prnText);
	Printer.Canvas.Font.Assign (MemoBoard.Font);
	for lin := 0 to MemoBoard.Lines.Count - 1 do
	  writeln (prnText, MemoBoard.Lines[lin]);
      finally
	System.Close (prnText);
      end;
    end;
  end;

  procedure TIXEngineForm.btnOkClick(Sender: TObject);
  begin
    Close;
  end;

  procedure TIXEngineForm.FormKeyDown(Sender: TObject; var Key: Word;
    Shift: TShiftState);
  begin
    if Key = Vk_Escape then
    begin
      Close;
      Key := 0;
    end;
  end;

  procedure TIXEngineForm.memBoardChange(Sender: TObject);
  begin
    MemoBoard.ShowHint := false;
  end;

  procedure TIXEngineForm.ShowException (Sender : TObject; E : Exception);
  begin
    if E is EIncrementalException then
    begin
      ExceptionToShow := E as EIncrementalException;
      ShowModal;
      ExceptionToShow := nil;
    end else
    if Assigned (FAppOnException) then
       FAppOnException (Sender, E)
    else
       Application.ShowException (E)
  end;

  procedure TIXEngineForm.RaiseIX (	 mcKind     : integer;
				   const msgrParams : array of const;
					 hc	    : THelpContext;
				   var	 IX	    : EIncrementalException);
  begin
    IX := EIncrementalException.Create (MessagerClass, mcKind, msgrParams, hc);
    raise IX;
  end;

  function  TIXEngineForm.RecastToIX (	    E	       : Exception;
					    mcKind     : integer;
				      const msgrParams : array of const;
					    hc	       : THelpContext) : boolean;
  var IX : EIncrementalException;
  begin
    Result := E is EIncrementalException;
    if Result then
      with E as EIncrementalException do
      Add (MessagerClass, mcKind, msgrParams, hc) else
    if Assigned (FOnRecast) and
       not FInRecastEvent then
    begin
      FInRecastEvent := true;
      try
	Result := OnRecast (E, IX);
      finally
	FInRecastEvent := false;
	if Assigned (IX) then
	  Result := RecastToIX (IX, mcKind, msgrParams, hc);
      end;
    end else
      RaiseIX (mcKind, msgrParams, hc, IX);
  end;

  procedure UnitExitProc; far;
  begin
    if Assigned (DefaultIXEngineForm) then
    begin
      DefaultIXEngineForm.Free;
      DefaultIXEngineForm := nil;
    end;
  end;

initialization {=================================================}
  DefaultIXEngineForm := TIXEngineForm.Create(nil);
  AddExitProc (UnitExitProc);
end.
